/* Generated automatically. */
static const char configuration_arguments[] = "/usr/obj/ports/arm-none-eabi-gcc-12.2.0-arm/gcc-12.2.0/configure --enable-languages=c,c++ --enable-multilib --enable-interwork --with-gmp=/usr/local --with-newlib --disable-libcc1 --enable-cpp --without-isl --without-zstd --target=arm-none-eabi --disable-shared --disable-nls --disable-werror --prefix=/usr/local --sysconfdir=/etc --mandir=/usr/local/man --infodir=/usr/local/info --localstatedir=/var --disable-silent-rules --disable-gtk-doc";
static const char thread_model[] = "single";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "cpu", "arm7tdmi" } };
